@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2018 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     This program is free software; you can redistribute it and/or modify
@@ -72,6 +72,7 @@ static struct pdf_sec {
    struct {
      int use_aes;
      int encrypt_metadata;
+     int need_adobe_extension;
    } setting;
 
    struct {
@@ -106,6 +107,7 @@ pdf_enc_init (int use_aes, int encrypt_metadata)
   srand(current_time); /* For AES IV */
   p->setting.use_aes = use_aes;
   p->setting.encrypt_metadata = encrypt_metadata;
+  p->setting.need_adobe_extension = 0;
 }
 
 #define PRODUCER \
@@ -463,17 +465,21 @@ getpass (const char *prompt)
 static void
 check_version (struct pdf_sec *p, int version)
 {
-  if (p->V > 2 && version < 4) {
+  if (p->V > 2 && version < 14) {
     WARN("Current encryption setting requires PDF version >= 1.4.");
     p->V = 1;
     p->key_size = 5;
-  } else if (p->V == 4 && version < 5) {
+  } else if (p->V == 4 && version < 15) {
     WARN("Current encryption setting requires PDF version >= 1.5.");
     p->V = 2;
-  } else if (p->V ==5 && version < 7) {
+  } else if (p->V ==5 && version < 17) {
     WARN("Current encryption setting requires PDF version >= 1.7" \
          " (plus Adobe Extension Level 3).");
     p->V = 4;
+    p->key_size = 16;
+  }
+  if (p->V == 5 && version < 20) {
+    p->setting.need_adobe_extension = 1;
   }
 }
 
@@ -786,7 +792,7 @@ pdf_encrypt_obj (void)
   }
 
 #ifdef USE_ADOBE_EXTENSION
-  if (p->R > 5) {
+  if (p->R > 5 && p->setting.need_adobe_extension != 0) {
     pdf_obj *catalog = pdf_doc_catalog();
     pdf_obj *ext  = pdf_new_dict();
     pdf_obj *adbe = pdf_new_dict();
