@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2007-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2007-2018 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -271,10 +271,10 @@ pdf_set_use_predictor (int bval)
   compression_use_predictor = bval ? 1 : 0;
 }
 
-static unsigned pdf_version = PDF_VERSION_DEFAULT;
+static int pdf_version = PDF_VERSION_DEFAULT;
 
 void
-pdf_set_version (unsigned version)
+pdf_set_version (int version)
 {
   /* Don't forget to update CIDFont_stdcc_def[] in cid.c too! */
   if (version >= PDF_VERSION_MIN && version <= PDF_VERSION_MAX) {
@@ -282,12 +282,30 @@ pdf_set_version (unsigned version)
   }
 }
 
-unsigned
+int
 pdf_get_version (void)
 {
   return pdf_version;
 }
 
+int
+pdf_get_version_major (void)
+{
+  return pdf_version/10;
+}
+
+int
+pdf_get_version_minor (void)
+{
+  return pdf_version%10;
+}
+
+int
+pdf_check_version (int major, int minor)
+{
+  return (pdf_version >= major*10+minor) ? 0 : -1;
+}
+
 int
 pdf_obj_get_verbose(void)
 {
@@ -329,7 +347,7 @@ pdf_out_init (const char *filename, int do_encryption, int enable_objstm)
   add_xref_entry(0, 0, 0, 0xffff);
   next_label = 1;
 
-  if (pdf_version >= 5) {
+  if (pdf_version >= 15) {
     if (enable_objstm) {
       xref_stream = pdf_new_stream(STREAM_COMPRESS);
       xref_stream->flags |= OBJ_NO_ENCRYPT;
@@ -362,8 +380,11 @@ pdf_out_init (const char *filename, int do_encryption, int enable_objstm)
         ERROR("Unable to open file.");
     }
   }
-  pdf_out(pdf_output_file, "%PDF-1.", strlen("%PDF-1."));
-  v = '0' + pdf_version;
+  pdf_out(pdf_output_file, "%PDF-", strlen("%PDF-"));
+  v = '0' + (pdf_version / 10);
+  pdf_out(pdf_output_file, &v, 1);
+  pdf_out(pdf_output_file, ".", 1);
+  v = '0' + (pdf_version % 10);
   pdf_out(pdf_output_file, &v, 1);
   pdf_out(pdf_output_file, "\n", 1);
   pdf_out(pdf_output_file, BINARY_MARKER, strlen(BINARY_MARKER));
@@ -510,7 +531,7 @@ pdf_out_flush (void)
     if (verbose) {
       if (compression_level > 0) {
 	MESG("Compression saved %ld bytes%s\n", compression_saved,
-	     pdf_version < 5 ? ". Try \"-V 5\" for better compression" : "");
+	     pdf_version < 15 ? ". Try \"-V 1.5\" for better compression" : "");
       }
     }
 #if !defined(LIBDPX)
@@ -3754,13 +3775,13 @@ pdf_open (const char *ident, FILE *file)
     pdf_obj *new_version;
     int version = check_for_pdf_version(file);
 
-    if (version < 1 || version > pdf_version) {
-      WARN("pdf_open: Not a PDF 1.[1-%u] file.", pdf_version);
-/*
-  Try to embed the PDF image, even if the PDF version is newer than
-  the setting.
-      return NULL;
-*/
+    if (version < 10)
+      WARN("Unrecognized PDF version specified for input PDF file: %d.%d",
+        pdf_version/10, pdf_version%10);
+    else if (version > pdf_version) {
+      WARN("Trying to include PDF file with version (%d.%d), which is " \
+           "newer than current output PDF setting (%d.%d).",
+        version/10, version%10, pdf_version/10, pdf_version%10);
     }
 
     pf = pdf_file_new(file);
@@ -3782,17 +3803,17 @@ pdf_open (const char *ident, FILE *file)
 
     new_version = pdf_deref_obj(pdf_lookup_dict(pf->catalog, "Version"));
     if (new_version) {
-      unsigned int minor;
+      unsigned int major, minor;
 
       if (!PDF_OBJ_NAMETYPE(new_version) ||
-	  sscanf(pdf_name_value(new_version), "1.%u", &minor) != 1) {
+          sscanf(pdf_name_value(new_version), "%u.%u", &major, &minor) != 2) {
 	pdf_release_obj(new_version);
 	WARN("Illegal Version entry in document catalog. Broken PDF file?");
 	goto error;
       }
 
-      if (pf->version < minor)
-	pf->version = minor;
+      if (pf->version < major*10+minor)
+        pf->version = major*10+minor;
 
       pdf_release_obj(new_version);
     }
@@ -3826,12 +3847,12 @@ pdf_files_close (void)
 static int
 check_for_pdf_version (FILE *file) 
 {
-  unsigned int minor;
+  unsigned int major, minor;
 
   rewind(file);
 
   return (ungetc(fgetc(file), file) == '%' &&
-	  fscanf(file, "%%PDF-1.%u", &minor) == 1) ? minor : -1;
+	  fscanf(file, "%%PDF-%u.%u", &major, &minor) == 2) ? (major*10+minor) : -1;
 }
 
 int
@@ -3845,8 +3866,6 @@ check_for_pdf (FILE *file)
   if (version <= pdf_version)
     return 1;
 
-  WARN("Version of PDF file (1.%d) is newer than version limit specification.",
-       version);
   return 1;
 }
 
@@ -3867,15 +3886,12 @@ import_dict (pdf_obj *key, pdf_obj *value, void *pdata)
   return 0;
 }
 
-static pdf_obj loop_marker = { PDF_OBJ_INVALID, 0, 0, 0, 0, NULL };
-
 static pdf_obj *
 pdf_import_indirect (pdf_obj *object)
 {
   pdf_file *pf = OBJ_FILE(object);
   unsigned int obj_num = OBJ_NUM(object);
   unsigned short obj_gen = OBJ_GEN(object);
-
   pdf_obj *ref;
 
   ASSERT(pf);
@@ -3886,11 +3902,9 @@ pdf_import_indirect (pdf_obj *object)
   }
 
   if ((ref = pf->xref_table[obj_num].indirect)) {
-    if (ref == &loop_marker)
-      ERROR("Loop in object hierarchy detected. Broken PDF file?");
     return  pdf_link_obj(ref);
   } else {
-    pdf_obj *obj, *tmp;
+    pdf_obj *obj, *reserved, *imported;
 
     obj = pdf_get_object(pf, obj_num, obj_gen);
     if (!obj) {
@@ -3898,14 +3912,36 @@ pdf_import_indirect (pdf_obj *object)
       return NULL;
     }
 
-    /* We mark the reference to be able to detect loops */
-    pf->xref_table[obj_num].indirect = &loop_marker;
-
-    tmp = pdf_import_object(obj);
-    
-    pf->xref_table[obj_num].indirect = ref = pdf_ref_obj(tmp);
-    
-    pdf_release_obj(tmp);
+    /* Fix for circular reference issue
+     *
+     * Older version of dvipdfmx disallowed the following case of
+     * circular reference:
+     *   obj #1 --> << /Kids [2 0 R] >>
+     *   obj #2 --> << /Parents [1 0 R] >>
+     * The problem is in that dvipdfmx gives new labels to objects after they
+     * are completely read.
+     * 
+     * Dirty hack though...
+     */
+    reserved = pdf_new_null(); /* for reservation of label */
+    pf->xref_table[obj_num].indirect = ref = pdf_new_ref(reserved);
+    imported = pdf_import_object(obj);
+    /* Substitute object here...
+     * We can't use pdf_transfer_label() here, since "reserved" object may
+     * have already be referenced by others and possibly be deref'ed later.
+     */
+    reserved->refcount = imported->refcount;
+    reserved->type     = imported->type;
+    reserved->flags    = imported->flags;
+    reserved->data     = imported->data;
+    /* Object returned by pdf_import_object(obj) might be simply a link to "obj".
+     * In that case we can't simply delete that object.
+     */
+    if (imported->refcount == 1) {
+      RELEASE(imported);
+      imported = NULL;
+    }
+    pdf_release_obj(reserved);
     pdf_release_obj(obj);
     
     return  pdf_link_obj(ref);
