@@ -1,5 +1,5 @@
 /* This is extractbb, a bounding box extraction program.
-    Copyright (C) 2008-2017 by Jin-Hwan Cho and Matthias Franz
+    Copyright (C) 2008-2018 by Jin-Hwan Cho and Matthias Franz
     and the dvipdfmx project team.
 
     This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 #include <time.h>
 #include <string.h>
 
+#include "dpxutil.h"
+
 #include "numbers.h"
 #include "system.h"
 #include "mem.h"
@@ -32,6 +34,7 @@
 #include "pdfobj.h"
 #include "pdfdoc.h"
 #include "pdfparse.h"
+#include "pdfdraw.h"
 
 #include "bmpimage.h"
 #include "jpegimage.h"
@@ -56,7 +59,7 @@ static int Include_Page = 1;
 static void show_version(void)
 {
   fprintf (stdout, "\nThis is %s Version " VERSION "\n", my_name);
-  fprintf (stdout, "\nCopyright (C) 2008-2017 by Jin-Hwan Cho and Matthias Franz\n");
+  fprintf (stdout, "\nCopyright (C) 2008-2018 by Jin-Hwan Cho and Matthias Franz\n");
   fprintf (stdout, "\nThis is free software; you can redistribute it and/or modify\n");
   fprintf (stdout, "it under the terms of the GNU General Public License as published by\n");
   fprintf (stdout, "the Free Software Foundation; either version 2 of the License, or\n");
@@ -177,7 +180,7 @@ static void write_xbb(char *fname,
     fprintf(fp, "%%%%HiResBoundingBox: %f %f %f %f\n",
             bbllx_f, bblly_f, bburx_f, bbury_f);
     if (pdf_version >= 0) {
-      fprintf(fp, "%%%%PDFVersion: 1.%d\n", pdf_version);
+      fprintf(fp, "%%%%PDFVersion: %d.%d\n", pdf_version/10, pdf_version%10);
       fprintf(fp, "%%%%Pages: %d\n", pagecount);
     }
   }
@@ -255,6 +258,8 @@ static void do_pdf (FILE *fp, char *filename)
   int page_no = Include_Page;
   int count;
   pdf_rect bbox;
+  pdf_tmatrix matrix;
+  pdf_coord   p1, p2, p3, p4;
 
   pf = pdf_open(filename, fp);
   if (!pf) {
@@ -262,7 +267,7 @@ static void do_pdf (FILE *fp, char *filename)
     return;
   }
   count = pdf_doc_get_page_count(pf);
-  page  = pdf_doc_get_page(pf, page_no, PageBox, &bbox, NULL);
+  page  = pdf_doc_get_page(pf, page_no, PageBox, &bbox, &matrix, NULL);
 
   pdf_close(pf);
 
@@ -270,6 +275,24 @@ static void do_pdf (FILE *fp, char *filename)
     return;
 
   pdf_release_obj(page);
+
+  /* Image's attribute "bbox" here is affected by /Rotate entry of included
+   * PDF page.
+   */
+  p1.x = bbox.llx; p1.y = bbox.lly;
+  pdf_dev_transform(&p1, &matrix);
+  p2.x = bbox.urx; p2.y = bbox.lly;
+  pdf_dev_transform(&p2, &matrix);
+  p3.x = bbox.urx; p3.y = bbox.ury;
+  pdf_dev_transform(&p3, &matrix);
+  p4.x = bbox.llx; p4.y = bbox.ury;
+  pdf_dev_transform(&p4, &matrix);
+
+  bbox.llx = min4(p1.x, p2.x, p3.x, p4.x);
+  bbox.lly = min4(p1.y, p2.y, p3.y, p4.y);
+  bbox.urx = max4(p1.x, p2.x, p3.x, p4.x);
+  bbox.ury = max4(p1.y, p2.y, p3.y, p4.y);
+
   write_xbb(filename, bbox.llx, bbox.lly, bbox.urx, bbox.ury,
             pdf_file_get_version(pf), count);
 }
@@ -398,179 +421,3 @@ int extractbb (int argc, char *argv[])
 
   return 0;
 }
-
-#if defined(LIBDPX)
-static void do_aptex_bmp (FILE *fp, char *filename, pdf_rect * box)
-{
-  int    width, height;
-  double xdensity, ydensity;
-
-  if (bmp_get_bbox(fp, &width, &height, &xdensity, &ydensity) < 0) {
-    WARN("%s does not look like a BMP file...\n", filename);
-    return;
-  }
-
-  box->llx = 0.0;
-  box->lly = 0.0;
-  box->urx = xdensity*width;
-  box->ury = ydensity*height;
-  return;
-}
-
-static void do_aptex_jpeg (FILE *fp, char *filename, pdf_rect * box)
-{
-  int    width, height;
-  double xdensity, ydensity;
-
-  if (jpeg_get_bbox(fp, &width, &height, &xdensity, &ydensity) < 0) {
-    WARN("%s does not look like a JPEG file...\n", filename);
-    return;
-  }
-
-  box->llx = 0.0;
-  box->lly = 0.0;
-  box->urx = xdensity*width;
-  box->ury = ydensity*height;
-  return;
-}
-
-static void do_aptex_jp2 (FILE *fp, char *filename, pdf_rect * box)
-{
-  int    width, height;
-  double xdensity, ydensity;
-
-  if (jp2_get_bbox(fp, &width, &height, &xdensity, &ydensity) < 0) {
-    WARN("%s does not look like a JP2/JPX file...\n", filename);
-    return;
-  }
-
-  box->llx = 0.0;
-  box->lly = 0.0;
-  box->urx = xdensity*width;
-  box->ury = ydensity*height;
-  return;
-}
-
-#ifdef HAVE_LIBPNG
-static void do_aptex_png (FILE *fp, char *filename, pdf_rect * box)
-{
-  uint32_t width, height;
-  double xdensity, ydensity;
-
-  if (png_get_bbox(fp, &width, &height, &xdensity, &ydensity) < 0) {
-    WARN("%s does not look like a PNG file...\n", filename);
-    return;
-  }
-
-  box->llx = 0.0;
-  box->lly = 0.0;
-  box->urx = xdensity*width;
-  box->ury = ydensity*height;
-  return;
-}
-#endif /* HAVE_LIBPNG */
-
-static void do_aptex_pdf (FILE *fp, char *filename, pdf_rect * box)
-{
-  pdf_obj *page;
-  pdf_file *pf;
-  int page_no = Include_Page;
-  int count;
-  pdf_rect bbox;
-
-  pf = pdf_open(filename, fp);
-  if (!pf) {
-    WARN("%s does not look like a PDF file...\n", filename);
-    return;
-  }
-  count = pdf_doc_get_page_count(pf);
-  page  = pdf_doc_get_page(pf, page_no, PageBox, &bbox, NULL);
-
-  pdf_close(pf);
-
-  if (!page)
-    return;
-
-  pdf_release_obj(page);
-
-  box->llx = bbox.llx;
-  box->lly = bbox.lly;
-  box->urx = bbox.urx;
-  box->ury = bbox.ury;
-}
-
-void aptex_extractbb (char * pict, uint32_t page, uint32_t rect, pdf_rect * bbox)
-{
-  FILE *infile = NULL;
-  char *kpse_file_name = NULL;
-  int    pictwd, pictht;
-  double xdensity, ydensity;
-
-  if (page == 0)
-    Include_Page = 1;
-  else
-    Include_Page = page;
-
-  PageBox = rect;
-
-  if (kpse_in_name_ok(pict))
-  {
-    infile = MFOPEN(pict, FOPEN_RBIN_MODE);
-    if (infile)
-    {
-      kpse_file_name = xstrdup(pict);
-    }
-    else
-    {
-      kpse_file_name = kpse_find_pict(pict);
-      if (kpse_file_name && kpse_in_name_ok(kpse_file_name))
-        infile = MFOPEN(kpse_file_name, FOPEN_RBIN_MODE);
-    }
-  }
-  if (infile == NULL)
-  {
-    WARN("Can't find file (%s), or it is forbidden to read ...skipping\n", pict);
-    goto cont;
-  }
-  if (check_for_bmp(infile))
-  {
-    do_aptex_bmp(infile, kpse_file_name, bbox);
-    goto cont;
-  }
-  if (check_for_jpeg(infile))
-  {
-    do_aptex_jpeg(infile, kpse_file_name, bbox);
-    goto cont;
-  }
-  if (check_for_jp2(infile))
-  {
-    do_aptex_jp2(infile, kpse_file_name, bbox);
-    goto cont;
-  }
-  if (check_for_pdf(infile))
-  {
-    pdf_files_init();
-    pdf_set_version(PDF_VERSION_MAX);
-    do_aptex_pdf(infile, kpse_file_name, bbox);
-    pdf_files_close();
-    goto cont;
-  }
-#ifdef HAVE_LIBPNG
-  if (check_for_png(infile))
-  {
-    do_aptex_png(infile, kpse_file_name, bbox);
-    goto cont;
-  }
-#endif /* HAVE_LIBPNG */
-  WARN("Can't handle file type for file named %s\n", pict);
-  bbox->llx = 0.0;
-  bbox->lly = 0.0;
-  bbox->urx = 0.0;
-  bbox->ury = 0.0;
-cont:
-  if (kpse_file_name)
-    RELEASE(kpse_file_name);
-  if (infile)
-    MFCLOSE(infile);
-}
-#endif /* LIBDPX */
