@@ -2,7 +2,7 @@
 ** FileSystem.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -36,7 +36,7 @@ using namespace std;
 
 #ifdef _WIN32
 	#include <direct.h>
-	#include <windows.h>
+	#include "windows.hpp"
 	const char *FileSystem::DEVNULL = "nul";
 	const char FileSystem::PATHSEP = '\\';
 	#define unlink _unlink
@@ -98,8 +98,8 @@ bool FileSystem::remove (const string &fname) {
  *  @param[in] remove_src remove file 'src' if true
  *  @return true on success */
 bool FileSystem::copy (const string &src, const string &dest, bool remove_src) {
-	ifstream ifs(src.c_str(), ios::in|ios::binary);
-	ofstream ofs(dest.c_str(), ios::out|ios::binary);
+	ifstream ifs(src, ios::in|ios::binary);
+	ofstream ofs(dest, ios::out|ios::binary);
 	if (ifs && ofs) {
 		ofs << ifs.rdbuf();
 		if (!ifs.fail() && !ofs.fail() && remove_src) {
@@ -272,7 +272,7 @@ bool FileSystem::rmdir (const string &dirname) {
 			struct dirent *ent;
 			while ((ent = readdir(dir)) && ok) {
 				const char *fname = ent->d_name;
-				string path = string(fname) + "/" + fname;
+				string path = dirname + "/" + fname;
 				if (isDirectory(path)) {
 					if (strcmp(fname, ".") != 0 && strcmp(fname, "..") != 0)
 						ok = rmdir(path) && s_rmdir(path);
@@ -310,7 +310,8 @@ bool FileSystem::exists (const string &fname) {
 bool FileSystem::isDirectory (const string &fname) {
 	if (const char *cfname = fname.c_str()) {
 #ifdef _WIN32
-		return GetFileAttributes(cfname) & FILE_ATTRIBUTE_DIRECTORY;
+		auto attr = GetFileAttributes(cfname);
+		return attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY);
 #else
 		struct stat attr;
 		return stat(cfname, &attr) == 0 && S_ISDIR(attr.st_mode);
@@ -335,16 +336,16 @@ bool FileSystem::isFile (const string &fname) {
 }
 
 
-int FileSystem::collect (const char *dirname, vector<string> &entries) {
+int FileSystem::collect (const std::string &dirname, vector<string> &entries) {
 	entries.clear();
 #ifdef _WIN32
-	string pattern = string(dirname) + "/*";
+	string pattern = dirname + "/*";
 	WIN32_FIND_DATA data;
 	HANDLE h = FindFirstFile(pattern.c_str(), &data);
 	bool ready = (h == INVALID_HANDLE_VALUE);
 	while (!ready) {
 		string fname = data.cFileName;
-		string path = string(dirname)+"/"+fname;
+		string path = dirname+"/"+fname;
 		string typechar = isFile(path) ? "f" : isDirectory(path) ? "d" : "?";
 		if (fname != "." && fname != "..")
 			entries.emplace_back(typechar+fname);
@@ -352,11 +353,11 @@ int FileSystem::collect (const char *dirname, vector<string> &entries) {
 	}
 	FindClose(h);
 #else
-	if (DIR *dir = opendir(dirname)) {
+	if (DIR *dir = opendir(dirname.c_str())) {
 		struct dirent *ent;
 		while ((ent = readdir(dir))) {
 			string fname = ent->d_name;
-			string path = string(dirname)+"/"+fname;
+			string path = dirname+"/"+fname;
 			string typechar = isFile(path) ? "f" : isDirectory(path) ? "d" : "?";
 			if (fname != "." && fname != "..")
 				entries.emplace_back(typechar+fname);
