@@ -2,7 +2,7 @@
 ** SVGOutput.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -33,8 +33,9 @@
 
 using namespace std;
 
-SVGOutput::SVGOutput (const char *base, const string &pattern, int zipLevel)
-	: _path(base ? base : ""), _pattern(pattern), _stdout(base == 0), _zipLevel(zipLevel), _page(-1)
+
+SVGOutput::SVGOutput (const string &base, const string &pattern, int zipLevel)
+	: _path(base), _pattern(pattern), _stdout(base.empty()), _zipLevel(zipLevel), _page(-1)
 {
 }
 
@@ -46,17 +47,24 @@ SVGOutput::SVGOutput (const char *base, const string &pattern, int zipLevel)
 ostream& SVGOutput::getPageStream (int page, int numPages) const {
 	string fname = filename(page, numPages);
 	if (fname.empty()) {
-		_osptr.reset();
-		return cout;
+		if (_zipLevel == 0) {
+			_osptr.reset();
+			return cout;
+		}
+#ifdef _WIN32
+		if (_setmode(_fileno(stdout), _O_BINARY) == -1)
+			throw MessageException("can't open stdout in binary mode");
+#endif
+		return *(_osptr = util::make_unique<ZLibOutputStream>(cout, ZLIB_GZIP, _zipLevel));
 	}
 	if (page == _page)
 		return *_osptr;
 
 	_page = page;
 	if (_zipLevel > 0)
-		_osptr.reset(new ZLibOutputStream(fname, _zipLevel));
+		_osptr = util::make_unique<ZLibOutputFileStream>(fname, ZLIB_GZIP, _zipLevel);
 	else
-		_osptr.reset(new ofstream(fname.c_str()));
+		_osptr = util::make_unique<ofstream>(fname);
 	if (!_osptr)
 		throw MessageException("can't open file "+fname+" for writing");
 	return *_osptr;
