@@ -2,7 +2,7 @@
 ** dvisvgm.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,11 +18,14 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <config.h>
+#include <algorithm>
 #include <clipper.hpp>
 #include <fstream>
 #include <iostream>
 #include <potracelib.h>
 #include <sstream>
+#include <vector>
 #include <xxhash.h>
 #include <zlib.h>
 #include "CommandLine.hpp"
@@ -34,18 +37,24 @@
 #include "Font.hpp"
 #include "FontEngine.hpp"
 #include "Ghostscript.hpp"
-#include "HtmlSpecialHandler.hpp"
+#include "HyperlinkManager.hpp"
 #include "Message.hpp"
 #include "PageSize.hpp"
+#include "PDFToSVG.hpp"
 #include "PSInterpreter.hpp"
 #include "PsSpecialHandler.hpp"
 #include "SignalHandler.hpp"
+#include "SourceInput.hpp"
 #include "SVGOutput.hpp"
 #include "System.hpp"
+#include "utility.hpp"
 #include "version.hpp"
 
 #ifndef DISABLE_WOFF
+#include <brotli/encode.h>
+#include <woff2/version.h>
 #include "ffwrapper.h"
+#include "TTFAutohint.hpp"
 #endif
 
 using namespace std;
@@ -61,10 +70,12 @@ static string remove_path (string fname) {
 }
 
 
-static string ensure_suffix (string fname, bool eps) {
-	size_t dotpos = remove_path(fname).rfind('.');
-	if (dotpos == string::npos)
-		fname += (eps ? ".eps" : ".dvi");
+static string ensure_suffix (string fname, const string &suffix) {
+	if (!fname.empty()) {
+		size_t dotpos = remove_path(fname).rfind('.');
+		if (dotpos == string::npos)
+			fname += "." + suffix;
+	}
 	return fname;
 }
 
@@ -111,7 +122,8 @@ static bool set_cache_dir (const CommandLine &args) {
 	if (args.cacheOpt.given() && args.cacheOpt.value().empty()) {
 		cout << "cache directory: " << (PhysicalFont::CACHE_PATH ? PhysicalFont::CACHE_PATH : "(none)") << '\n';
 		try {
-			FontCache::fontinfo(PhysicalFont::CACHE_PATH, cout, true);
+			if (PhysicalFont::CACHE_PATH)
+				FontCache::fontinfo(PhysicalFont::CACHE_PATH, cout, true);
 		}
 		catch (StreamReaderException &e) {
 			Message::wstream(true) << "failed reading cache data\n";
@@ -135,79 +147,140 @@ static bool set_temp_dir (const CommandLine &args) {
 }
 
 
-static bool check_bbox (const string &bboxstr) {
-	const char *formats[] = {"none", "min", "preview", "papersize", "dvi", 0};
-	for (const char **p=formats; *p; ++p)
-		if (bboxstr == *p)
-			return true;
+static void check_bbox (const string &bboxstr) {
+	for (const char *fmt : {"none", "min", "preview", "papersize", "dvi"})
+		if (bboxstr == fmt)
+			return;
 	if (isalpha(bboxstr[0])) {
 		try {
 			PageSize size(bboxstr);
-			return true;
 		}
 		catch (const PageSizeException &e) {
-			Message::estream(true) << "invalid bounding box format '" << bboxstr << "'\n";
-			return false;
+			throw MessageException("invalid bounding box format '" + bboxstr + "'");
 		}
 	}
-	try {
-		// check if given bbox argument is valid, i.e. doesn't throw an exception
-		BoundingBox bbox;
-		bbox.set(bboxstr);
-		return true;
-	}
-	catch (const MessageException &e) {
-		Message::estream(true) << e.what() << '\n';
-		return false;
-	}
+	// check if given bbox argument is valid, i.e. doesn't throw an exception
+	BoundingBox bbox;
+	bbox.set(bboxstr);
 }
 
 
-static void print_version (bool extended) {
-	ostringstream oss;
-	oss << "dvisvgm " << PROGRAM_VERSION;
-	if (extended) {
-		if (strlen(TARGET_SYSTEM) > 0)
-			oss << " (" TARGET_SYSTEM ")";
-		int len = oss.str().length();
-		oss << "\n" << string(len, '-') << "\n"
-			"clipper:     " << CLIPPER_VERSION "\n";
-#ifndef DISABLE_WOFF
-		oss << "fontforge:   " << ff_version() << '\n';
-#endif
-		oss << "freetype:    " << FontEngine::version() << "\n";
+// Helper class to generate a list of version information of the used libraries.
+class VersionInfo {
+	public:
+		void add (const string &name, const string &version, bool ignoreEmpty=false) {
+			if (!version.empty() || !ignoreEmpty)
+				append(name, util::trim(version));
+		}
 
-		Ghostscript gs;
-		string gsver = gs.revision(true);
-		if (!gsver.empty())
-			oss << "Ghostscript: " << gsver + "\n";
-		const unsigned xxh_ver = XXH_versionNumber();
-		oss <<
-#ifdef MIKTEX
-			"MiKTeX:      " << FileFinder::instance().version() << "\n"
-#else
-			"kpathsea:    " << FileFinder::instance().version() << "\n"
+		void add (const string &name, const char *version, bool ignoreEmpty=false) {
+			if (version && *version)
+				append(name, util::trim(version));
+			else if (!ignoreEmpty)
+				append(name, "");
+		}
+
+		void add (const string &name, const vector<int> &versionComponents) {
+			string version;
+			for (auto it=versionComponents.begin(); it != versionComponents.end(); ++it) {
+				if (it != versionComponents.begin())
+					version += '.';
+				version += to_string(*it);
+			}
+			append(name, version);
+		}
+
+		/** Adds a version number given as a single unsigned integer, and optionally
+		 *  extracts its components, e.g. 0x00010203 => "1.2.3" (3 components separated
+		 *  by multiples of 256).
+		 *  @param[in] name library name
+		 *  @param[in] version version number
+		 *  @param[in] compcount number of components the version consists of
+		 *  @param[in] factor factor used to separate the components */
+		void add (const string &name, uint32_t version, int compcount=1, uint32_t factor=0xffffffff) {
+			string str;
+			while (compcount-- > 0) {
+				if (!str.empty())
+					str.insert(0, ".");
+				str.insert(0, to_string(version % factor));
+				version /= factor;
+			}
+			append(name, str);
+		}
+
+		/** Writes the version information to the given output stream. */
+		void write (ostream &os) {
+			using Entry = pair<string,string>;
+			sort(_versionPairs.begin(), _versionPairs.end(), [](const Entry &e1, const Entry &e2) {
+				return util::tolower(e1.first) < util::tolower(e2.first);
+			});
+			size_t maxNameLength=0;
+			for (const Entry &versionPair : _versionPairs)
+				maxNameLength = max(maxNameLength, versionPair.first.length());
+			for (const Entry &versionPair : _versionPairs) {
+				string name = versionPair.first+":";
+				os << left << setw(maxNameLength+2) << name;
+				os << (versionPair.second.empty() ? "unknown" : versionPair.second) << '\n';
+			}
+		}
+
+	protected:
+		void append (const string &name, const string &version) {
+			_versionPairs.emplace_back(pair<string,string>(name, version));
+		}
+
+	private:
+		vector<pair<string,string>> _versionPairs;
+};
+
+
+static void print_version (bool extended) {
+	string versionstr = string(PROGRAM_NAME)+" "+PROGRAM_VERSION;
+#ifdef TARGET_SYSTEM
+	if (extended && strlen(TARGET_SYSTEM) > 0)
+		versionstr += " (" TARGET_SYSTEM ")";
 #endif
-			"potrace:     " << (strchr(potrace_version(), ' ') ? strchr(potrace_version(), ' ')+1 : "unknown") << "\n"
-			"xxhash:      " << xxh_ver/10000 << '.' << (xxh_ver/100)%100 << '.' << xxh_ver%100 << "\n"
-			"zlib:        " << zlibVersion();
+	cout << versionstr << '\n';
+	if (extended) {
+		cout << string(versionstr.length(), '-') << '\n';
+		VersionInfo versionInfo;
+		versionInfo.add("clipper", CLIPPER_VERSION);
+		versionInfo.add("freetype", FontEngine::version());
+		versionInfo.add("potrace", strchr(potrace_version(), ' '));
+		versionInfo.add("xxhash", XXH_versionNumber(), 3, 100);
+		versionInfo.add("zlib", zlibVersion());
+		versionInfo.add("Ghostscript", Ghostscript().revisionstr(), true);
+#ifndef DISABLE_WOFF
+		versionInfo.add("brotli", BrotliEncoderVersion(), 3, 0x1000);
+		versionInfo.add("woff2", woff2::version, 3, 0x100);
+		versionInfo.add("fontforge", ff_version());
+		versionInfo.add("ttfautohint", TTFAutohint().version(), true);
+#endif
+#ifdef MIKTEX
+		versionInfo.add("MiKTeX", FileFinder::instance().version());
+#else
+		versionInfo.add("kpathsea", FileFinder::instance().version());
+#endif
+		versionInfo.write(cout);
 	}
-	cout << oss.str() << endl;
 }
 
 
 static void init_fontmap (const CommandLine &cmdline) {
-	const char *mapseq = cmdline.fontmapOpt.given() ? cmdline.fontmapOpt.value().c_str() : 0;
-	bool additional = mapseq && strchr("+-=", *mapseq);
-	if (!mapseq || additional) {
-		const char *mapfiles[] = {"ps2pk.map", "dvipdfm.map", "psfonts.map", 0};
+	string mapseq;
+	if (cmdline.fontmapOpt.given())
+		mapseq = cmdline.fontmapOpt.value();
+	bool additional = !mapseq.empty() && strchr("+-=", mapseq[0]);
+	if (mapseq.empty() || additional) {
 		bool found = false;
-		for (const char **p=mapfiles; *p && !found; p++)
-			found = FontMap::instance().read(*p);
+		for (string mapfile : {"ps2pk", "pdftex", "dvipdfm", "psfonts"}) {
+			if ((found = FontMap::instance().read(mapfile+".map")))
+				break;
+		}
 		if (!found)
 			Message::wstream(true) << "none of the default map files could be found\n";
 	}
-	if (mapseq)
+	if (!mapseq.empty())
 		FontMap::instance().read(mapseq);
 }
 
@@ -248,8 +321,8 @@ static void set_variables (const CommandLine &cmdline) {
 
 
 int main (int argc, char *argv[]) {
-	CommandLine cmdline;
 	try {
+		CommandLine cmdline;
 		cmdline.parse(argc, argv);
 		if (argc == 1 || cmdline.helpOpt.given()) {
 			cmdline.help(cout, cmdline.helpOpt.value());
@@ -268,47 +341,47 @@ int main (int argc, char *argv[]) {
 		}
 		if (!set_cache_dir(cmdline) || !set_temp_dir(cmdline))
 			return 0;
-		if (cmdline.stdoutOpt.given() && cmdline.zipOpt.given()) {
-			Message::estream(true) << "writing SVGZ files to stdout is not supported\n";
-			return 1;
-		}
-		if (!check_bbox(cmdline.bboxOpt.value()))
-			return 1;
-		if (!HtmlSpecialHandler::setLinkMarker(cmdline.linkmarkOpt.value()))
+		check_bbox(cmdline.bboxOpt.value());
+		if (!HyperlinkManager::setLinkMarker(cmdline.linkmarkOpt.value()))
 			Message::wstream(true) << "invalid argument '"+cmdline.linkmarkOpt.value()+"' supplied for option --linkmark\n";
-		if (argc > 1 && cmdline.filenames().size() < 1) {
-			Message::estream(true) << "no input file given\n";
-			return 1;
+		if (cmdline.stdinOpt.given() || cmdline.singleDashGiven()) {
+			if (!cmdline.filenames().empty())
+				throw MessageException("option - or --stdin can't be used together with a filename");
+			cmdline.addFilename("");  // empty filename => read from stdin
 		}
-	}
-	catch (MessageException &e) {
-		Message::estream() << e.what() << '\n';
-		return 1;
-	}
+		if (argc > 1 && cmdline.filenames().empty())
+			throw MessageException("no input file given");
+
+		SignalHandler::instance().start();
+		string inputfile = ensure_suffix(cmdline.filenames()[0],
+			cmdline.epsOpt.given() ? "eps" : cmdline.pdfOpt.given() ? "pdf" : "dvi");
+		SourceInput srcin(inputfile);
+		if (!srcin.getInputStream(true))
+			throw MessageException("can't open file '" + srcin.getMessageFileName() + "' for reading");
 
-	bool eps_given = cmdline.epsOpt.given();
-	string inputfile = ensure_suffix(cmdline.filenames()[0], eps_given);
-	ifstream ifs(inputfile.c_str(), ios::binary|ios::in);
-	if (!ifs) {
-		Message::estream(true) << "can't open file '" << inputfile << "' for reading\n";
-		return 0;
-	}
-	try {
 		double start_time = System::time();
 		set_variables(cmdline);
-		SVGOutput out(cmdline.stdoutOpt.given() ? nullptr : inputfile.c_str(),
+		SVGOutput out(cmdline.stdoutOpt.given() ? "" : srcin.getFileName(),
 			cmdline.outputOpt.value(),
 			cmdline.zipOpt.given() ? cmdline.zipOpt.value() : 0);
-		SignalHandler::instance().start();
-		if (cmdline.epsOpt.given()) {
-			EPSToSVG eps2svg(inputfile, out);
-			eps2svg.convert();
+		if (cmdline.epsOpt.given() || cmdline.pdfOpt.given()) {
+			auto img2svg = unique_ptr<ImageToSVG>(
+				cmdline.epsOpt.given()
+				? static_cast<ImageToSVG*>(new EPSToSVG(srcin.getFilePath(), out))
+				: static_cast<ImageToSVG*>(new PDFToSVG(srcin.getFilePath(), out)));
+			img2svg->convert();
+			Message::mstream().indent(0);
+			Message::mstream(false, Message::MC_PAGE_NUMBER) << "file converted in " << (System::time()-start_time) << " seconds\n";
+		}
+		else if (cmdline.pdfOpt.given()) {
+			PDFToSVG pdf2svg(srcin.getFilePath(), out);
+			pdf2svg.convert();
 			Message::mstream().indent(0);
 			Message::mstream(false, Message::MC_PAGE_NUMBER) << "file converted in " << (System::time()-start_time) << " seconds\n";
 		}
 		else {
 			init_fontmap(cmdline);
-			DVIToSVG dvi2svg(ifs, out);
+			DVIToSVG dvi2svg(srcin.getInputStream(), out);
 			const char *ignore_specials=nullptr;
 			if (cmdline.noSpecialsOpt.given())
 				ignore_specials = cmdline.noSpecialsOpt.value().empty() ? "*" : cmdline.noSpecialsOpt.value().c_str();
