@@ -1,6 +1,6 @@
 /* progname.c: the executable name we were invoked as; general initialization.
 
-   Copyright 1994, 1996, 1997, 2008-2013, 2016 Karl Berry.
+   Copyright 1994, 1996, 1997, 2008-2013, 2016-2018 Karl Berry.
    Copyright 1998-2005 Olaf Weber.
 
    This library is free software; you can redistribute it and/or
@@ -153,10 +153,12 @@ CopyFirst (register char *a, char *b)
   strcat (a, StripFirst (b));
 }
 
-/* Returns NULL on error.  Prints intermediate results if global
-   `ll_verbose' is nonzero.  */
+/* Returns NULL on error, such as an unresolvable symlink.  Prints
+   intermediate results if global `ll_verbose' is nonzero.  Otherwise,
+   returns a pointer to a static buffer (sorry).  */
 
-#define EX(s)           (strlen (s) && strcmp (s, "/") ? "/" : "")
+#define EMPTY_STRING(s) (*(s) == 0)
+#define EX(s)           (!EMPTY_STRING (s) && strcmp (s, "/") ? "/" : "")
 #define EXPOS           EX(post)
 #define EXPRE           EX(pre)
 
@@ -170,30 +172,14 @@ expand_symlinks (kpathsea kpse, char *s)
   struct stat st;
   int done;
 
-  /* Check for symlink loops.  It's difficult to check for all the
-     possibilities ourselves, so let the kernel do it.  And make it
-     conditional so that people can see where the infinite loop is
-     being caused (see engtools#1536).  */
-  /* There used to be a test for a variable |ll_loop| here, but
-     it was initialized to zero and never updated */
-  if (0) {
-    FILE *f = fopen (s, "r");
-    if (!f && errno == ELOOP) {
-      /* Not worried about other errors, we'll get to them in due course.  */
-      perror (s);
-      return NULL;
-    }
-    if (f) fclose (f);
-  }
-
   strcpy (post, s);
   strcpy (pre, "");
 
-  while (strlen (post) != 0) {
+  while (!EMPTY_STRING (post)) {
     CopyFirst (pre, post);
 
     if (lstat (pre, &st) != 0) {
-      fprintf (stderr, "lstat(%s) failed ...\n", pre);
+      fprintf (stderr, "lstat(%s) failed: ", pre);
       perror (pre);
       return NULL;
     }
@@ -209,7 +195,7 @@ expand_symlinks (kpathsea kpse, char *s)
       } else {
         a = pre[0];     /* handle links through the root */
         strcpy (tmp, StripLast (pre));
-        if (!strlen (pre) && a == '/')
+        if (EMPTY_STRING (pre) && a == '/')
           strcpy (pre, "/");
 
         if (kpse->ll_verbose) {
@@ -223,7 +209,7 @@ expand_symlinks (kpathsea kpse, char *s)
         a = pre[0];     /* handle links through the root */
         while (!strncmp (sym, "..", 2)
                && (sym[2] == 0 || sym[2] == '/')
-               && strlen (pre) != 0
+               && !EMPTY_STRING (pre)
                && strcmp (pre, ".")
                && strcmp (pre, "..")
                && (strlen (pre) < 3
@@ -241,11 +227,11 @@ expand_symlinks (kpathsea kpse, char *s)
           else
             printf ("%s == %s%s%s\n", before, pre, EXPOS, post);
         }
-        if (!strlen (pre) && a == '/')
+        if (EMPTY_STRING (pre) && a == '/')
           strcpy (pre, "/");
       }
 
-      if (strlen (post) != 0 && strlen (sym) != 0)
+      if (!EMPTY_STRING (post) && !EMPTY_STRING (sym))
         strcat (sym, "/");
 
       strcat (sym, post);
@@ -326,14 +312,14 @@ remove_dots (kpathsea kpse, string dir)
 }
 
 /* Return directory ARGV0 comes from.  Check PATH if ARGV0 is not
-   absolute.  */
+   absolute.  If ARGV0 cannot be found (e.g., --progname=nonesuch), quit.  */
 
 string
 kpathsea_selfdir (kpathsea kpse, const_string argv0)
 {
-  string self = NULL;
   string name;
   string ret;
+  string self = NULL;
 
   if (kpathsea_absolute_p (kpse, argv0, true)) {
     self = xstrdup (argv0);
@@ -393,16 +379,23 @@ kpathsea_selfdir (kpathsea kpse, const_string argv0)
   if (!self)
     self = concat3 (".", DIR_SEP_STRING, argv0);
 
-  name = remove_dots (kpse, expand_symlinks (kpse, self));
+  /* If we can't expand symlinks (--progname=nonesuch), give up.  */
+  name = expand_symlinks (kpse, self);
+  if (!name) {
+    fprintf (stderr, "kpathsea: Can't get directory of program name: %s\n",
+             self);
+    exit (1);
+  }
+
+  /* If we have something real, we can resolve ./ and ../ elements.  */
+  name = remove_dots (kpse, name);
 
 #ifndef AMIGA
   free (self);
 #endif
 
   ret = xdirname (name);
-
   free (name);
-
   return ret;
 }
 
@@ -503,7 +496,7 @@ kpathsea_set_program_name (kpathsea kpse,  const_string argv0,
 #if defined(WIN32)
   if (!kpse->File_system_codepage)
     kpse->File_system_codepage = AreFileApisANSI() ? GetACP() : GetOEMCP();
-  cp = kpse->File_system_codepage;
+  cp = kpse->Win32_codepage = kpse->File_system_codepage;
   if (cp == 932 || cp == 936 || cp == 950) {
     kpse->Is_cp932_system = cp;
   }
