@@ -1,5 +1,5 @@
 /*
-Copyright 1996-2014 Han The Thanh, <thanh@pdftex.org>
+Copyright 1996-2016 Han The Thanh, <thanh@pdftex.org>
 
 This file is part of pdfTeX.
 
@@ -66,10 +66,12 @@ extern "C" {
 #include <pdftexdir/ptexmac.h>
 #include <pdftexdir/pdftex-common.h>
 
-// This function from pdftex.web gets declared in pdftexcoerce.h in the
+// These functions from pdftex.web gets declared in pdftexcoerce.h in the
 // usual web2c way, but we cannot include that file here because C++
 // does not allow it.
 extern int getpdfsuppresswarningpagegroup(void);
+extern integer getpdfsuppressptexinfo(void);
+extern integer zround(double);
 }
 
 // The prefix "PTEX" for the PDF keys is special to pdfTeX;
@@ -77,6 +79,11 @@ extern int getpdfsuppresswarningpagegroup(void);
 
 #define pdfkeyprefix "PTEX"
 
+#define MASK_SUPPRESS_PTEX_FULLBANNER 0x01
+#define MASK_SUPPRESS_PTEX_FILENAME   0x02
+#define MASK_SUPPRESS_PTEX_PAGENUMBER 0x04
+#define MASK_SUPPRESS_PTEX_INFODICT   0x08
+
 // PdfObject encapsulates the xpdf Object type,
 // and properly frees its resources on destruction.
 // Use obj-> to access members of the Object,
@@ -159,6 +166,16 @@ static XRef *xref = 0;
 static PdfDocument *find_add_document(char *file_name)
 {
     PdfDocument *p = pdfDocuments;
+#ifdef _WIN32
+    int file_name_is_changed = 0;
+    wchar_t *fnamew;
+    if (file_system_codepage == CP_UTF8 && win32_codepage != CP_UTF8) {
+        fnamew = get_wstring_from_mbstring(CP_UTF8, file_name, fnamew=NULL);
+        file_name = get_mbstring_from_wstring(win32_codepage, fnamew, file_name=NULL);
+        file_name_is_changed = 1;
+        xfree(fnamew);
+    }
+#endif
     while (p && strcmp(p->file_name, file_name) != 0)
         p = p->next;
     if (p) {
@@ -178,6 +195,10 @@ static PdfDocument *find_add_document(char *file_name)
     p->inObjList = 0;
     p->next = pdfDocuments;
     pdfDocuments = p;
+#ifdef _WIN32
+    if (file_name_is_changed == 1)
+        xfree(file_name);
+#endif
     return p;
 }
 
@@ -275,6 +296,7 @@ static int addInObj(InObjType type, Ref ref, fd_entry * fd, int e)
     return n->num;
 }
 
+#if 0 /* unusewd */
 static int getNewObjectNumber(Ref ref)
 {
     InObj *p;
@@ -294,6 +316,7 @@ static int getNewObjectNumber(Ref ref)
     return -60000;
 #endif
 }
+#endif
 
 static void copyObject(Object *);
 
@@ -405,22 +428,24 @@ static void copyFont(char *tag, Object * fontRef)
     // Only handle included Type1 (and Type1C) fonts; anything else will be copied.
     // Type1C fonts are replaced by Type1 fonts, if REPLACE_TYPE1C is true.
     if (!fixedinclusioncopyfont && fontRef->fetch(xref, &fontdict)->isDict()
-        && fontdict->dictLookup((char *)"Subtype", &subtype)->isName()
+        && fontdict->dictLookup("Subtype", &subtype)->isName()
         && !strcmp(subtype->getName(), "Type1")
-        && fontdict->dictLookup((char *)"BaseFont", &basefont)->isName()
-        && fontdict->dictLookupNF((char *)"FontDescriptor", &fontdescRef)->isRef()
+        && fontdict->dictLookup("BaseFont", &basefont)->isName()
+        && fontdict->dictLookupNF("FontDescriptor", &fontdescRef)->isRef()
         && fontdescRef->fetch(xref, &fontdesc)->isDict()
-        && (fontdesc->dictLookup((char *)"FontFile", &fontfile)->isStream()
+        && (fontdesc->dictLookup("FontFile", &fontfile)->isStream()
             || (REPLACE_TYPE1C
-                && fontdesc->dictLookup((char *)"FontFile3", &fontfile)->isStream()
-                && fontfile->streamGetDict()->lookup((char *)"Subtype",
+                && fontdesc->dictLookup("FontFile3", &fontfile)->isStream()
+                && fontfile->streamGetDict()->lookup("Subtype",
                                                      &ffsubtype)->isName()
                 && !strcmp(ffsubtype->getName(), "Type1C")))
         && (fontmap = lookup_fontmap(basefont->getName())) != NULL) {
-        // copy the value of /StemV
-        fontdesc->dictLookup((char *)"StemV", &stemV);
-        fd = epdf_create_fontdescriptor(fontmap, stemV->getInt());
-        if (fontdesc->dictLookup((char *)"CharSet", &charset) &&
+        // round /StemV value, since the PDF input is a float
+        // (see Font Descriptors in PDF reference), but we only store an
+        // integer, since we don't want to change the struct.
+        fontdesc->dictLookup("StemV", &stemV);
+        fd = epdf_create_fontdescriptor(fontmap, zround(stemV->getNum()));
+        if (fontdesc->dictLookup("CharSet", &charset) &&
             charset->isString() && is_subsetable(fontmap))
             epdf_mark_glyphs(fd, charset->getString()->getCString());
         else
@@ -838,6 +863,7 @@ void write_epdf(void)
     int rotate;
     double scale[6] = { 0, 0, 0, 0, 0, 0 };
     bool writematrix = false;
+    int suppress_ptex_info = getpdfsuppressptexinfo();
     static const char *pageDictKeys[] = {
         "LastModified",
         "Metadata",
@@ -865,15 +891,21 @@ void write_epdf(void)
     pdf_puts("/FormType 1\n");
 
     // write additional information
-    pdf_printf("/%s.FileName (%s)\n", pdfkeyprefix,
-               convertStringToPDFString(pdf_doc->file_name,
-                                        strlen(pdf_doc->file_name)));
-    pdf_printf("/%s.PageNumber %i\n", pdfkeyprefix, epdf_selected_page);
-    pdf_doc->doc->getDocInfoNF(&info);
-    if (info.isRef()) {
-        // the info dict must be indirect (PDF Ref p. 61)
-        pdf_printf("/%s.InfoDict ", pdfkeyprefix);
-        pdf_printf("%d 0 R\n", addOther(info.getRef()));
+    if ((suppress_ptex_info & MASK_SUPPRESS_PTEX_FILENAME) == 0) {
+        pdf_printf("/%s.FileName (%s)\n", pdfkeyprefix,
+                   convertStringToPDFString(pdf_doc->file_name,
+                                            strlen(pdf_doc->file_name)));
+    }
+    if ((suppress_ptex_info & MASK_SUPPRESS_PTEX_PAGENUMBER) == 0) {
+        pdf_printf("/%s.PageNumber %i\n", pdfkeyprefix, (int) epdf_selected_page);
+    }
+    if ((suppress_ptex_info & MASK_SUPPRESS_PTEX_INFODICT) == 0) {
+        pdf_doc->doc->getDocInfoNF(&info);
+        if (info.isRef()) {
+            // the info dict must be indirect (PDF Ref p. 61)
+            pdf_printf("/%s.InfoDict ", pdfkeyprefix);
+            pdf_printf("%d 0 R\n", addOther(info.getRef()));
+        }
     }
     // get the pagebox (media, crop...) to use.
     pagebox = get_pagebox(page, epdf_page_box);
@@ -924,13 +956,13 @@ void write_epdf(void)
     pdf_puts(stripzeros(s));
 
     // Metadata validity check (as a stream it must be indirect)
-    pageDict->lookupNF((char *)"Metadata", &dictObj);
+    pageDict->lookupNF("Metadata", &dictObj);
     if (!dictObj->isNull() && !dictObj->isRef())
         pdftex_warn("PDF inclusion: /Metadata must be indirect object");
 
     // copy selected items in Page dictionary except Resources & Group
     for (i = 0; pageDictKeys[i] != NULL; i++) {
-        pageDict->lookupNF((char *)pageDictKeys[i], &dictObj);
+        pageDict->lookupNF(pageDictKeys[i], &dictObj);
         if (!dictObj->isNull()) {
             pdf_newline();
             pdf_printf("/%s ", pageDictKeys[i]);
@@ -939,7 +971,7 @@ void write_epdf(void)
     } 
 
     // handle page group
-    pageDict->lookupNF((char *)"Group", &dictObj);
+    pageDict->lookupNF("Group", &dictObj);
     if (!dictObj->isNull()) {
         if (pdfpagegroupval == 0) { 
             // another pdf with page group was included earlier on the
@@ -954,12 +986,12 @@ void write_epdf(void)
             copyObject(&dictObj);
         } else {
             // write Group dict as a separate object, since the Page dict also refers to it
-            pageDict->lookup((char *) "Group", &dictObj);
+            pageDict->lookup("Group", &dictObj);
             if (!dictObj->isDict())
                 pdftex_fail("PDF inclusion: /Group dict missing");
             writeSepGroup = true;
             initDictFromDict(groupDict, page->getGroup());
-            pdf_printf("/Group %d 0 R\n", pdfpagegroupval);
+            pdf_printf("/Group %ld 0 R\n", (long)pdfpagegroupval);
         }
     }
 
@@ -1003,21 +1035,21 @@ void write_epdf(void)
 
         // Variant B: copy stream without recompressing
         //
-        contents->streamGetDict()->lookup((char *)"F", &obj1);
+        contents->streamGetDict()->lookup("F", &obj1);
         if (!obj1->isNull()) {
             pdftex_fail("PDF inclusion: Unsupported external stream");
         }
-        contents->streamGetDict()->lookup((char *)"Length", &obj1);
+        contents->streamGetDict()->lookup("Length", &obj1);
         assert(!obj1->isNull());
         pdf_puts("/Length ");
         copyObject(&obj1);
         pdf_puts("\n");
-        contents->streamGetDict()->lookup((char *)"Filter", &obj1);
+        contents->streamGetDict()->lookup("Filter", &obj1);
         if (!obj1->isNull()) {
             pdf_puts("/Filter ");
             copyObject(&obj1);
             pdf_puts("\n");
-            contents->streamGetDict()->lookup((char *)"DecodeParms", &obj1);
+            contents->streamGetDict()->lookup("DecodeParms", &obj1);
             if (!obj1->isNull()) {
                 pdf_puts("/DecodeParms ");
                 copyObject(&obj1);
