@@ -11,12 +11,19 @@
 #define setintone(w,a) ((w).cint1=(a))
 #endif
 
+#ifndef KANJI
 #define KANJI
-
-#if !defined(WIN32)
-extern int sjisterminal;
 #endif
 
+/* allow file names with 0x5c in (e)pTeX on windows */
+#if defined(WIN32)
+#include <kpathsea/knj.h>
+#define not_kanji_char_seq(a,b) (!(is_cp932_system && isknj(a) && isknj2(b)))
+#else
+#define not_kanji_char_seq(a,b) (1)
+#endif
+#define notkanjicharseq not_kanji_char_seq
+
 /* functions */
 #define Hi(x) (((x) >> 8) & 0xff)
 #define Lo(x) ((x) & 0xff)
